Class: Tina4::ResponseCache::CacheEntry

Inherits:
Struct
  • Object
show all
Defined in:
lib/tina4/response_cache.rb

Overview

vary / vary_values carry the RFC 9111 s4.1 bookkeeping: the field names the origin nominated, and the values they had on the request that caused this response to be stored.

Instance Attribute Summary collapse

Instance Attribute Details

#bodyObject

Returns the value of attribute body

Returns:

  • (Object)

    the current value of body



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def body
  @body
end

#content_typeObject

Returns the value of attribute content_type

Returns:

  • (Object)

    the current value of content_type



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def content_type
  @content_type
end

#expires_atObject

Returns the value of attribute expires_at

Returns:

  • (Object)

    the current value of expires_at



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def expires_at
  @expires_at
end

#status_codeObject

Returns the value of attribute status_code

Returns:

  • (Object)

    the current value of status_code



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def status_code
  @status_code
end

#varyObject

Returns the value of attribute vary

Returns:

  • (Object)

    the current value of vary



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def vary
  @vary
end

#vary_valuesObject

Returns the value of attribute vary_values

Returns:

  • (Object)

    the current value of vary_values



43
44
45
# File 'lib/tina4/response_cache.rb', line 43

def vary_values
  @vary_values
end