Class: Tina4::ORM
Class Method Summary collapse
- .all(limit: nil, offset: nil, order_by: nil, include: nil) ⇒ Object
-
.auto_map ⇒ Object
Auto-map flag (no-op in Ruby since snake_case is native).
- .auto_map=(val) ⇒ Object
-
.belongs_to(name, class_name: nil, foreign_key: nil) ⇒ Object
belongs_to :user, class_name: “User”, foreign_key: “user_id”.
- .count(conditions = nil, params = []) ⇒ Object
- .create(attributes = {}) ⇒ Object
- .create_table ⇒ Object
- .db ⇒ Object
-
.db=(database) ⇒ Object
Per-model database binding.
-
.eager_load(instances, include_list) ⇒ Object
Eager load relationships for a collection of instances (prevents N+1).
-
.field_mapping ⇒ Object
Field mapping: { ‘db_column’ => ‘ruby_attribute’ }.
- .field_mapping=(map) ⇒ Object
- .find(id_or_filter = nil, filter = nil, **kwargs) ⇒ Object
- .find_or_fail(id) ⇒ Object
- .from_hash(hash) ⇒ Object
-
.has_many(name, class_name: nil, foreign_key: nil) ⇒ Object
has_many :posts, class_name: “Post”, foreign_key: “user_id”.
-
.has_one(name, class_name: nil, foreign_key: nil) ⇒ Object
has_one :profile, class_name: “Profile”, foreign_key: “user_id”.
-
.query ⇒ Tina4::QueryBuilder
Create a fluent QueryBuilder pre-configured for this model’s table and database.
-
.relationship_definitions ⇒ Object
Relationship definitions.
- .scope(name, filter_sql, params = []) ⇒ Object
- .select(sql, params = [], limit: nil, offset: nil, include: nil) ⇒ Object
- .select_one(sql, params = [], include: nil) ⇒ Object
-
.soft_delete ⇒ Object
Soft delete configuration.
- .soft_delete=(val) ⇒ Object
- .soft_delete_field ⇒ Object
- .soft_delete_field=(val) ⇒ Object
- .where(conditions, params = [], include: nil) ⇒ Object
- .with_trashed(conditions = "1=1", params = [], limit: 20, offset: 0) ⇒ Object
Instance Method Summary collapse
- #delete ⇒ Object
- #errors ⇒ Object
- #force_delete ⇒ Object
-
#initialize(attributes = {}) ⇒ ORM
constructor
A new instance of ORM.
-
#load(sql, params = [], include: nil) ⇒ Object
Load a record into this instance via select_one.
- #persisted? ⇒ Boolean
- #query_belongs_to(related_class, foreign_key: nil) ⇒ Object (also: #imperative_belongs_to)
- #query_has_many(related_class, foreign_key: nil, limit: 100, offset: 0) ⇒ Object (also: #imperative_has_many)
-
#query_has_one(related_class, foreign_key: nil) ⇒ Object
(also: #imperative_has_one)
── Imperative relationship methods (ad-hoc, like Python/PHP/Node) ──.
- #restore ⇒ Object
- #save ⇒ Object
- #select(*fields) ⇒ Object
- #to_array ⇒ Object (also: #to_list)
-
#to_h(include: nil) ⇒ Object
(also: #to_hash, #to_dict, #to_assoc, #to_object)
Convert to hash using Ruby attribute names.
- #to_json(include: nil, **_args) ⇒ Object
- #to_s ⇒ Object
- #validate ⇒ Object
Methods included from FieldTypes
Constructor Details
#initialize(attributes = {}) ⇒ ORM
Returns a new instance of ORM.
375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 |
# File 'lib/tina4/orm.rb', line 375 def initialize(attributes = {}) @persisted = false @errors = [] @relationship_cache = {} attributes.each do |key, value| setter = "#{key}=" __send__(setter, value) if respond_to?(setter) end # Set defaults self.class.field_definitions.each do |name, opts| if __send__(name).nil? && opts[:default] __send__("#{name}=", opts[:default]) end end end |
Class Method Details
.all(limit: nil, offset: nil, order_by: nil, include: nil) ⇒ Object
232 233 234 235 236 237 238 239 240 241 242 |
# File 'lib/tina4/orm.rb', line 232 def all(limit: nil, offset: nil, order_by: nil, include: nil) sql = "SELECT * FROM #{table_name}" if soft_delete sql += " WHERE #{soft_delete_field} IS NULL OR #{soft_delete_field} = 0" end sql += " ORDER BY #{order_by}" if order_by results = db.fetch(sql, [], limit: limit, offset: offset) instances = results.map { |row| from_hash(row) } eager_load(instances, include) if include instances end |
.auto_map ⇒ Object
Auto-map flag (no-op in Ruby since snake_case is native)
56 57 58 |
# File 'lib/tina4/orm.rb', line 56 def auto_map @auto_map || false end |
.auto_map=(val) ⇒ Object
60 61 62 |
# File 'lib/tina4/orm.rb', line 60 def auto_map=(val) @auto_map = val end |
.belongs_to(name, class_name: nil, foreign_key: nil) ⇒ Object
belongs_to :user, class_name: “User”, foreign_key: “user_id”
96 97 98 99 100 101 102 103 104 105 106 |
# File 'lib/tina4/orm.rb', line 96 def belongs_to(name, class_name: nil, foreign_key: nil) relationship_definitions[name] = { type: :belongs_to, class_name: class_name || name.to_s.split("_").map(&:capitalize).join, foreign_key: foreign_key || "#{name}_id" } define_method(name) do load_belongs_to(name) end end |
.count(conditions = nil, params = []) ⇒ Object
256 257 258 259 260 261 262 263 264 265 266 |
# File 'lib/tina4/orm.rb', line 256 def count(conditions = nil, params = []) sql = "SELECT COUNT(*) as cnt FROM #{table_name}" where_parts = [] if soft_delete where_parts << "(#{soft_delete_field} IS NULL OR #{soft_delete_field} = 0)" end where_parts << "(#{conditions})" if conditions sql += " WHERE #{where_parts.join(' AND ')}" unless where_parts.empty? result = db.fetch_one(sql, params) result[:cnt].to_i end |
.create(attributes = {}) ⇒ Object
268 269 270 271 272 |
# File 'lib/tina4/orm.rb', line 268 def create(attributes = {}) instance = new(attributes) instance.save instance end |
.create_table ⇒ Object
286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 |
# File 'lib/tina4/orm.rb', line 286 def create_table return true if db.table_exists?(table_name) type_map = { integer: "INTEGER", string: "VARCHAR(255)", text: "TEXT", float: "REAL", decimal: "REAL", boolean: "INTEGER", date: "DATE", datetime: "DATETIME", timestamp: "TIMESTAMP", blob: "BLOB", json: "TEXT" } col_defs = [] field_definitions.each do |name, opts| sql_type = type_map[opts[:type]] || "TEXT" if opts[:type] == :string && opts[:length] sql_type = "VARCHAR(#{opts[:length]})" end parts = ["#{name} #{sql_type}"] parts << "PRIMARY KEY" if opts[:primary_key] parts << "AUTOINCREMENT" if opts[:auto_increment] parts << "NOT NULL" if !opts[:nullable] && !opts[:primary_key] if opts[:default] && !opts[:auto_increment] default_val = opts[:default].is_a?(String) ? "'#{opts[:default]}'" : opts[:default] parts << "DEFAULT #{default_val}" end col_defs << parts.join(" ") end sql = "CREATE TABLE IF NOT EXISTS #{table_name} (#{col_defs.join(', ')})" db.execute(sql) db.commit true end |
.db ⇒ Object
20 21 22 |
# File 'lib/tina4/orm.rb', line 20 def db @db || Tina4.database || auto_discover_db end |
.db=(database) ⇒ Object
Per-model database binding
25 26 27 |
# File 'lib/tina4/orm.rb', line 25 def db=(database) @db = database end |
.eager_load(instances, include_list) ⇒ Object
Eager load relationships for a collection of instances (prevents N+1). include is an array of relationship names, supporting dot notation for nesting.
143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 |
# File 'lib/tina4/orm.rb', line 143 def eager_load(instances, include_list) return if instances.nil? || instances.empty? # Group includes: top-level and nested top_level = {} include_list.each do |inc| parts = inc.to_s.split(".", 2) rel_name = parts[0].to_sym top_level[rel_name] ||= [] top_level[rel_name] << parts[1] if parts.length > 1 end top_level.each do |rel_name, nested| rel = relationship_definitions[rel_name] next unless rel klass = Object.const_get(rel[:class_name]) pk = primary_key_field || :id case rel[:type] when :has_one, :has_many fk = rel[:foreign_key] || "#{name.split('::').last.downcase}_id" pk_values = instances.map { |inst| inst.__send__(pk) }.compact.uniq next if pk_values.empty? placeholders = pk_values.map { "?" }.join(",") sql = "SELECT * FROM #{klass.table_name} WHERE #{fk} IN (#{placeholders})" results = klass.db.fetch(sql, pk_values) = results.map { |row| klass.from_hash(row) } # Eager load nested klass.eager_load(, nested) unless nested.empty? # Group by FK grouped = {} .each do |record| fk_val = record.__send__(fk.to_sym) if record.respond_to?(fk.to_sym) (grouped[fk_val] ||= []) << record end instances.each do |inst| pk_val = inst.__send__(pk) records = grouped[pk_val] || [] if rel[:type] == :has_one inst.instance_variable_get(:@relationship_cache)[rel_name] = records.first else inst.instance_variable_get(:@relationship_cache)[rel_name] = records end end when :belongs_to fk = rel[:foreign_key] || "#{rel_name}_id" fk_values = instances.map { |inst| inst.respond_to?(fk.to_sym) ? inst.__send__(fk.to_sym) : nil }.compact.uniq next if fk_values.empty? = klass.primary_key_field || :id placeholders = fk_values.map { "?" }.join(",") sql = "SELECT * FROM #{klass.table_name} WHERE #{} IN (#{placeholders})" results = klass.db.fetch(sql, fk_values) = results.map { |row| klass.from_hash(row) } klass.eager_load(, nested) unless nested.empty? lookup = {} .each { |r| lookup[r.__send__()] = r } instances.each do |inst| fk_val = inst.respond_to?(fk.to_sym) ? inst.__send__(fk.to_sym) : nil inst.instance_variable_get(:@relationship_cache)[rel_name] = lookup[fk_val] end end end end |
.field_mapping ⇒ Object
Field mapping: { ‘db_column’ => ‘ruby_attribute’ }
47 48 49 |
# File 'lib/tina4/orm.rb', line 47 def field_mapping @field_mapping || {} end |
.field_mapping=(map) ⇒ Object
51 52 53 |
# File 'lib/tina4/orm.rb', line 51 def field_mapping=(map) @field_mapping = map end |
.find(id_or_filter = nil, filter = nil, **kwargs) ⇒ Object
118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 |
# File 'lib/tina4/orm.rb', line 118 def find(id_or_filter = nil, filter = nil, **kwargs) include_list = kwargs.delete(:include) # find(id) — find by primary key # find(filter_hash) — find by criteria # find(name: "Alice") — keyword args as filter hash result = if id_or_filter.is_a?(Hash) find_by_filter(id_or_filter) elsif filter.is_a?(Hash) find_by_filter(filter) elsif !kwargs.empty? find_by_filter(kwargs) else find_by_id(id_or_filter) end if include_list && result instances = result.is_a?(Array) ? result : [result] eager_load(instances, include_list) end result end |
.find_or_fail(id) ⇒ Object
274 275 276 277 278 |
# File 'lib/tina4/orm.rb', line 274 def find_or_fail(id) result = find(id) raise "#{name} with #{primary_key_field || :id}=#{id} not found" if result.nil? result end |
.from_hash(hash) ⇒ Object
333 334 335 336 337 338 339 340 341 342 343 344 |
# File 'lib/tina4/orm.rb', line 333 def from_hash(hash) instance = new mapping_reverse = field_mapping.invert hash.each do |key, value| # Apply field mapping (db_col => ruby_attr) attr_name = mapping_reverse[key.to_s] || key setter = "#{attr_name}=" instance.__send__(setter, value) if instance.respond_to?(setter) end instance.instance_variable_set(:@persisted, true) instance end |
.has_many(name, class_name: nil, foreign_key: nil) ⇒ Object
has_many :posts, class_name: “Post”, foreign_key: “user_id”
83 84 85 86 87 88 89 90 91 92 93 |
# File 'lib/tina4/orm.rb', line 83 def has_many(name, class_name: nil, foreign_key: nil) relationship_definitions[name] = { type: :has_many, class_name: class_name || name.to_s.sub(/s$/, "").split("_").map(&:capitalize).join, foreign_key: foreign_key } define_method(name) do load_has_many(name) end end |
.has_one(name, class_name: nil, foreign_key: nil) ⇒ Object
has_one :profile, class_name: “Profile”, foreign_key: “user_id”
70 71 72 73 74 75 76 77 78 79 80 |
# File 'lib/tina4/orm.rb', line 70 def has_one(name, class_name: nil, foreign_key: nil) relationship_definitions[name] = { type: :has_one, class_name: class_name || name.to_s.split("_").map(&:capitalize).join, foreign_key: foreign_key } define_method(name) do load_has_one(name) end end |
.query ⇒ Tina4::QueryBuilder
Create a fluent QueryBuilder pre-configured for this model’s table and database.
Usage:
results = User.query.where("active = ?", [1]).order_by("name").get
114 115 116 |
# File 'lib/tina4/orm.rb', line 114 def query QueryBuilder.from(table_name, db: db) end |
.relationship_definitions ⇒ Object
Relationship definitions
65 66 67 |
# File 'lib/tina4/orm.rb', line 65 def relationship_definitions @relationship_definitions ||= {} end |
.scope(name, filter_sql, params = []) ⇒ Object
327 328 329 330 331 |
# File 'lib/tina4/orm.rb', line 327 def scope(name, filter_sql, params = []) define_singleton_method(name) do |limit: 20, offset: 0| where(filter_sql, params) end end |
.select(sql, params = [], limit: nil, offset: nil, include: nil) ⇒ Object
244 245 246 247 248 249 |
# File 'lib/tina4/orm.rb', line 244 def select(sql, params = [], limit: nil, offset: nil, include: nil) results = db.fetch(sql, params, limit: limit, offset: offset) instances = results.map { |row| from_hash(row) } eager_load(instances, include) if include instances end |
.select_one(sql, params = [], include: nil) ⇒ Object
251 252 253 254 |
# File 'lib/tina4/orm.rb', line 251 def select_one(sql, params = [], include: nil) results = select(sql, params, limit: 1, include: include) results.first end |
.soft_delete ⇒ Object
Soft delete configuration
30 31 32 |
# File 'lib/tina4/orm.rb', line 30 def soft_delete @soft_delete || false end |
.soft_delete=(val) ⇒ Object
34 35 36 |
# File 'lib/tina4/orm.rb', line 34 def soft_delete=(val) @soft_delete = val end |
.soft_delete_field ⇒ Object
38 39 40 |
# File 'lib/tina4/orm.rb', line 38 def soft_delete_field @soft_delete_field || :is_deleted end |
.soft_delete_field=(val) ⇒ Object
42 43 44 |
# File 'lib/tina4/orm.rb', line 42 def soft_delete_field=(val) @soft_delete_field = val end |
.where(conditions, params = [], include: nil) ⇒ Object
219 220 221 222 223 224 225 226 227 228 229 230 |
# File 'lib/tina4/orm.rb', line 219 def where(conditions, params = [], include: nil) sql = "SELECT * FROM #{table_name}" if soft_delete sql += " WHERE (#{soft_delete_field} IS NULL OR #{soft_delete_field} = 0) AND (#{conditions})" else sql += " WHERE #{conditions}" end results = db.fetch(sql, params) instances = results.map { |row| from_hash(row) } eager_load(instances, include) if include instances end |
.with_trashed(conditions = "1=1", params = [], limit: 20, offset: 0) ⇒ Object
280 281 282 283 284 |
# File 'lib/tina4/orm.rb', line 280 def with_trashed(conditions = "1=1", params = [], limit: 20, offset: 0) sql = "SELECT * FROM #{table_name} WHERE #{conditions}" results = db.fetch(sql, params, limit: limit, offset: offset) results.map { |row| from_hash(row) } end |
Instance Method Details
#delete ⇒ Object
423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 |
# File 'lib/tina4/orm.rb', line 423 def delete pk = self.class.primary_key_field || :id pk_value = __send__(pk) return false unless pk_value self.class.db.transaction do |db| if self.class.soft_delete db.update( self.class.table_name, { self.class.soft_delete_field => 1 }, { pk => pk_value } ) else db.delete(self.class.table_name, { pk => pk_value }) end end @persisted = false true end |
#errors ⇒ Object
505 506 507 |
# File 'lib/tina4/orm.rb', line 505 def errors @errors end |
#force_delete ⇒ Object
443 444 445 446 447 448 449 450 451 452 453 |
# File 'lib/tina4/orm.rb', line 443 def force_delete pk = self.class.primary_key_field || :id pk_value = __send__(pk) raise "Cannot delete: no primary key value" unless pk_value self.class.db.transaction do |db| db.delete(self.class.table_name, { pk => pk_value }) end @persisted = false true end |
#load(sql, params = [], include: nil) ⇒ Object
Load a record into this instance via select_one. Returns true if found and loaded, false otherwise.
486 487 488 489 490 491 492 493 494 495 496 497 498 499 |
# File 'lib/tina4/orm.rb', line 486 def load(sql, params = [], include: nil) @relationship_cache = {} result = self.class.select_one(sql, params, include: include) return false unless result mapping_reverse = self.class.field_mapping.invert result.to_h.each do |key, value| attr_name = mapping_reverse[key.to_s] || key setter = "#{attr_name}=" __send__(setter, value) if respond_to?(setter) end @persisted = true true end |
#persisted? ⇒ Boolean
501 502 503 |
# File 'lib/tina4/orm.rb', line 501 def persisted? @persisted end |
#query_belongs_to(related_class, foreign_key: nil) ⇒ Object Also known as: imperative_belongs_to
669 670 671 672 673 674 675 |
# File 'lib/tina4/orm.rb', line 669 def query_belongs_to(, foreign_key: nil) fk = foreign_key || "#{.name.split('::').last.downcase}_id" fk_value = respond_to?(fk.to_sym) ? __send__(fk.to_sym) : nil return nil unless fk_value .find(fk_value) end |
#query_has_many(related_class, foreign_key: nil, limit: 100, offset: 0) ⇒ Object Also known as: imperative_has_many
656 657 658 659 660 661 662 663 664 665 666 667 |
# File 'lib/tina4/orm.rb', line 656 def query_has_many(, foreign_key: nil, limit: 100, offset: 0) pk = self.class.primary_key_field || :id pk_value = __send__(pk) return [] unless pk_value fk = foreign_key || "#{self.class.name.split('::').last.downcase}_id" results = .db.fetch( "SELECT * FROM #{.table_name} WHERE #{fk} = ?", [pk_value], limit: limit, offset: offset ) results.map { |row| .from_hash(row) } end |
#query_has_one(related_class, foreign_key: nil) ⇒ Object Also known as: imperative_has_one
── Imperative relationship methods (ad-hoc, like Python/PHP/Node) ──
644 645 646 647 648 649 650 651 652 653 654 |
# File 'lib/tina4/orm.rb', line 644 def query_has_one(, foreign_key: nil) pk = self.class.primary_key_field || :id pk_value = __send__(pk) return nil unless pk_value fk = foreign_key || "#{self.class.name.split('::').last.downcase}_id" result = .db.fetch_one( "SELECT * FROM #{.table_name} WHERE #{fk} = ?", [pk_value] ) result ? .from_hash(result) : nil end |
#restore ⇒ Object
455 456 457 458 459 460 461 462 463 464 465 466 467 468 469 470 471 |
# File 'lib/tina4/orm.rb', line 455 def restore raise "Model does not support soft delete" unless self.class.soft_delete pk = self.class.primary_key_field || :id pk_value = __send__(pk) raise "Cannot restore: no primary key value" unless pk_value self.class.db.transaction do |db| db.update( self.class.table_name, { self.class.soft_delete_field => 0 }, { pk => pk_value } ) end __send__("#{self.class.soft_delete_field}=", 0) if respond_to?("#{self.class.soft_delete_field}=") true end |
#save ⇒ Object
391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 418 419 420 421 |
# File 'lib/tina4/orm.rb', line 391 def save @errors = [] @relationship_cache = {} # Clear relationship cache on save validate_fields return false unless @errors.empty? data = to_db_hash(exclude_nil: true) pk = self.class.primary_key_field || :id pk_value = __send__(pk) self.class.db.transaction do |db| if @persisted && pk_value filter = { pk => pk_value } data.delete(pk) # Remove mapped primary key too mapped_pk = self.class.field_mapping[pk.to_s] data.delete(mapped_pk.to_sym) if mapped_pk db.update(self.class.table_name, data, filter) else result = db.insert(self.class.table_name, data) if result[:last_id] && respond_to?("#{pk}=") __send__("#{pk}=", result[:last_id]) end @persisted = true end end self rescue => e @errors << e. false end |
#select(*fields) ⇒ Object
562 563 564 565 566 567 |
# File 'lib/tina4/orm.rb', line 562 def select(*fields) fields_str = fields.map(&:to_s).join(", ") pk = self.class.primary_key_field || :id pk_value = __send__(pk) self.class.db.fetch_one("SELECT #{fields_str} FROM #{self.class.table_name} WHERE #{pk} = ?", [pk_value]) end |
#to_array ⇒ Object Also known as: to_list
548 549 550 |
# File 'lib/tina4/orm.rb', line 548 def to_array to_h.values end |
#to_h(include: nil) ⇒ Object Also known as: to_hash, to_dict, to_assoc, to_object
Convert to hash using Ruby attribute names. Optionally include relationships via the include keyword.
511 512 513 514 515 516 517 518 519 520 521 522 523 524 525 526 527 528 529 530 531 532 533 534 535 536 537 538 539 540 541 |
# File 'lib/tina4/orm.rb', line 511 def to_h(include: nil) hash = {} self.class.field_definitions.each_key do |name| hash[name] = __send__(name) end if include # Group includes: top-level and nested top_level = {} include.each do |inc| parts = inc.to_s.split(".", 2) rel_name = parts[0].to_sym top_level[rel_name] ||= [] top_level[rel_name] << parts[1] if parts.length > 1 end top_level.each do |rel_name, nested| next unless self.class.relationship_definitions.key?(rel_name) = __send__(rel_name) if .nil? hash[rel_name] = nil elsif .is_a?(Array) hash[rel_name] = .map { |r| r.to_h(include: nested.empty? ? nil : nested) } else hash[rel_name] = .to_h(include: nested.empty? ? nil : nested) end end end hash end |
#to_json(include: nil, **_args) ⇒ Object
554 555 556 |
# File 'lib/tina4/orm.rb', line 554 def to_json(include: nil, **_args) JSON.generate(to_h(include: include)) end |
#to_s ⇒ Object
558 559 560 |
# File 'lib/tina4/orm.rb', line 558 def to_s "#<#{self.class.name} #{to_h}>" end |
#validate ⇒ Object
473 474 475 476 477 478 479 480 481 482 |
# File 'lib/tina4/orm.rb', line 473 def validate errors = [] self.class.field_definitions.each do |name, opts| value = __send__(name) if !opts[:nullable] && value.nil? && !opts[:auto_increment] && !opts[:default] errors << "#{name} cannot be null" end end errors end |