Class: Tina4::ResponseCache::CacheEntry
- Inherits:
-
Struct
- Object
- Struct
- Tina4::ResponseCache::CacheEntry
- Defined in:
- lib/tina4/response_cache.rb
Overview
vary / vary_values carry the RFC 9111 s4.1 bookkeeping: the field names the origin nominated, and the values they had on the request that caused this response to be stored.
Instance Attribute Summary collapse
-
#body ⇒ Object
Returns the value of attribute body.
-
#content_type ⇒ Object
Returns the value of attribute content_type.
-
#expires_at ⇒ Object
Returns the value of attribute expires_at.
-
#status_code ⇒ Object
Returns the value of attribute status_code.
-
#vary ⇒ Object
Returns the value of attribute vary.
-
#vary_values ⇒ Object
Returns the value of attribute vary_values.
Instance Attribute Details
#body ⇒ Object
Returns the value of attribute body
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def body @body end |
#content_type ⇒ Object
Returns the value of attribute content_type
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def content_type @content_type end |
#expires_at ⇒ Object
Returns the value of attribute expires_at
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def expires_at @expires_at end |
#status_code ⇒ Object
Returns the value of attribute status_code
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def status_code @status_code end |
#vary ⇒ Object
Returns the value of attribute vary
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def vary @vary end |
#vary_values ⇒ Object
Returns the value of attribute vary_values
43 44 45 |
# File 'lib/tina4/response_cache.rb', line 43 def vary_values @vary_values end |