Class: Tina4::WebSocketConnection
- Inherits:
-
Object
- Object
- Tina4::WebSocketConnection
- Defined in:
- lib/tina4/websocket.rb
Instance Attribute Summary collapse
-
#id ⇒ Object
readonly
Returns the value of attribute id.
-
#on_close_handler ⇒ Object
Returns the value of attribute on_close_handler.
-
#on_error_handler ⇒ Object
Returns the value of attribute on_error_handler.
-
#on_message_handler ⇒ Object
Returns the value of attribute on_message_handler.
-
#params ⇒ Object
Returns the value of attribute params.
-
#path ⇒ Object
Returns the value of attribute path.
-
#rooms ⇒ Object
readonly
Returns the value of attribute rooms.
Instance Method Summary collapse
-
#broadcast(message, include_self: false) ⇒ Object
Broadcast a message to all other connections on the same path.
- #broadcast_to_room(room_name, message, exclude_self: false) ⇒ Object
- #build_frame(opcode, data) ⇒ Object
- #close(code: 1000, reason: "") ⇒ Object
-
#initialize(id, socket, ws_server: nil, path: "/") ⇒ WebSocketConnection
constructor
A new instance of WebSocketConnection.
- #join_room(room_name) ⇒ Object
- #leave_room(room_name) ⇒ Object
-
#on_close(&block) ⇒ Object
Register a close handler (decorator style, matching Python).
-
#on_message(&block) ⇒ Object
Register a message handler (decorator style, matching Python).
- #read_frame ⇒ Object
- #send(message) ⇒ Object (also: #send_text)
- #send_pong(data) ⇒ Object
Constructor Details
#initialize(id, socket, ws_server: nil, path: "/") ⇒ WebSocketConnection
Returns a new instance of WebSocketConnection.
221 222 223 224 225 226 227 228 229 230 231 |
# File 'lib/tina4/websocket.rb', line 221 def initialize(id, socket, ws_server: nil, path: "/") @id = id @socket = socket @params = {} @ws_server = ws_server @path = path @rooms = Set.new @on_message_handler = nil @on_close_handler = nil @on_error_handler = nil end |
Instance Attribute Details
#id ⇒ Object (readonly)
Returns the value of attribute id.
218 219 220 |
# File 'lib/tina4/websocket.rb', line 218 def id @id end |
#on_close_handler ⇒ Object
Returns the value of attribute on_close_handler.
219 220 221 |
# File 'lib/tina4/websocket.rb', line 219 def on_close_handler @on_close_handler end |
#on_error_handler ⇒ Object
Returns the value of attribute on_error_handler.
219 220 221 |
# File 'lib/tina4/websocket.rb', line 219 def on_error_handler @on_error_handler end |
#on_message_handler ⇒ Object
Returns the value of attribute on_message_handler.
219 220 221 |
# File 'lib/tina4/websocket.rb', line 219 def @on_message_handler end |
#params ⇒ Object
Returns the value of attribute params.
219 220 221 |
# File 'lib/tina4/websocket.rb', line 219 def params @params end |
#path ⇒ Object
Returns the value of attribute path.
219 220 221 |
# File 'lib/tina4/websocket.rb', line 219 def path @path end |
#rooms ⇒ Object (readonly)
Returns the value of attribute rooms.
218 219 220 |
# File 'lib/tina4/websocket.rb', line 218 def rooms @rooms end |
Instance Method Details
#broadcast(message, include_self: false) ⇒ Object
Broadcast a message to all other connections on the same path
261 262 263 264 265 266 267 268 269 |
# File 'lib/tina4/websocket.rb', line 261 def broadcast(, include_self: false) return unless @ws_server @ws_server.connections.each do |cid, conn| next if !include_self && cid == @id next if conn.path != @path conn.send_text() end end |
#broadcast_to_room(room_name, message, exclude_self: false) ⇒ Object
253 254 255 256 257 258 |
# File 'lib/tina4/websocket.rb', line 253 def broadcast_to_room(room_name, , exclude_self: false) return unless @ws_server exclude = exclude_self ? @id : nil @ws_server.broadcast_to_room(room_name, , exclude: exclude) end |
#build_frame(opcode, data) ⇒ Object
324 325 326 |
# File 'lib/tina4/websocket.rb', line 324 def build_frame(opcode, data) Tina4.build_frame(opcode, data) end |
#close(code: 1000, reason: "") ⇒ Object
288 289 290 291 292 293 |
# File 'lib/tina4/websocket.rb', line 288 def close(code: 1000, reason: "") payload = [code].pack("n") + reason frame = build_frame(0x8, payload) @socket.write(frame) rescue nil @socket.close rescue nil end |
#join_room(room_name) ⇒ Object
243 244 245 246 |
# File 'lib/tina4/websocket.rb', line 243 def join_room(room_name) @rooms.add(room_name) @ws_server&.join_room_for(@id, room_name) end |
#leave_room(room_name) ⇒ Object
248 249 250 251 |
# File 'lib/tina4/websocket.rb', line 248 def leave_room(room_name) @rooms.delete(room_name) @ws_server&.leave_room_for(@id, room_name) end |
#on_close(&block) ⇒ Object
Register a close handler (decorator style, matching Python).
239 240 241 |
# File 'lib/tina4/websocket.rb', line 239 def on_close(&block) @on_close_handler = block end |
#on_message(&block) ⇒ Object
Register a message handler (decorator style, matching Python).
234 235 236 |
# File 'lib/tina4/websocket.rb', line 234 def (&block) @on_message_handler = block end |
#read_frame ⇒ Object
295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 |
# File 'lib/tina4/websocket.rb', line 295 def read_frame first_byte = @socket.getbyte return nil unless first_byte opcode = first_byte & 0x0F second_byte = @socket.getbyte return nil unless second_byte masked = (second_byte & 0x80) != 0 length = second_byte & 0x7F if length == 126 length = @socket.read(2).unpack1("n") elsif length == 127 length = @socket.read(8).unpack1("Q>") end mask_key = masked ? @socket.read(4).bytes : nil data = @socket.read(length) || "" if masked && mask_key data = data.bytes.each_with_index.map { |b, i| b ^ mask_key[i % 4] }.pack("C*") end { opcode: opcode, data: data } rescue IOError, EOFError nil end |
#send(message) ⇒ Object Also known as: send_text
271 272 273 274 275 276 277 |
# File 'lib/tina4/websocket.rb', line 271 def send() data = .encode("UTF-8") frame = build_frame(0x1, data) @socket.write(frame) rescue IOError # Connection closed end |
#send_pong(data) ⇒ Object
281 282 283 284 285 286 |
# File 'lib/tina4/websocket.rb', line 281 def send_pong(data) frame = build_frame(0xA, data || "") @socket.write(frame) rescue IOError # Connection closed end |